/* This file is part Tempura: Interval Temporal Logic interpreter.
 *  
 * Copyright (C) 1987-2013  Roger Hale, Ben Moszkowski, Antonio Cau 
 *
 * Tempura is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Tempura is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Changes made by  AC: Antonio Cau
 *                 BCM: Ben Moszkowski
 *
 * $Id: r-symbol.c 150 2013-04-11 14:14:53Z acau $
 */


#include "system.h"




void noop(e)
NODEPTR e;
{
    error_break1 ("I'm sorry, Dave, I can't do \"%s\"", print_name (e));
}

void assert_stmt(e)
NODEPTR e;
{
    error_break1 ("\"%s\" can only be executed as a statement",print_name(e));
}

void assert_expr(e)
NODEPTR e;
{
    error_break1 ("\"%s\" cannot be executed as a statement", print_name(e));
    result (True);
}

/* 06/09/2001 added */
void assert_stmt_to_true(e)
NODEPTR e;
{
    NODEPTR e1;
    e1 = reduce_val (e);
    if (Ready) {
	if (e1 != True)
	  error_break0 ("Statement has reduced to false");
	e = True;
    };
    result(e);
}

void assert_true(e)
NODEPTR e;
{
    NODEPTR e1;
    e = reduce_val (e);
    if (!Ready) {
        switch (node_type(e)) {
            case NOT:
	        switch (node_type(e1 = node_lnk_1(e)) ) {
		    case OR:
	                Retry = T;
		        e = node2 (AND,
		                   node1 (NOT, node_lnk_1(e1)),
			           node1 (NOT, node_lnk_2(e1)));
                        break;
		    case AND:
	                Retry = T;
		        e = node2 (OR,
		                   node1 (NOT, node_lnk_1(e1)),
			           node1 (NOT, node_lnk_2(e1)));
                        break;
		    case NOT:
	                Retry = T;
		        e = node_lnk_1(e1);
		        break;
                    default:
		        e1 = assign_curr (node_lnk_1(e), False);
			if (Ready)
			    e = e1;
	                break;
                }
	        break;
	    default:
	        e1 = assign_curr (e, True);
		if (Ready)
		    e = e1;
	        break;
        }
    }
    result (e);
}


/*******************\
*                   *
*  tempura symbols  *
*                   *
\*******************/

void reduce_const(e)
NODEPTR e;
{
    Ready = val_defined (e);
}

void reduce_id_val(e)
NODEPTR e;
{
    NODEPTR loc;
    NODEPTR val;
    if (DebugLevel>2) {
        printf("***reduce_id_val, NF=%1d, PF=%d, MD=%1d\n",NextFlag,PrevFlag,mem_defined1(adj_loc1 (mem_loc (e))));
        show_mem_all1(adj_loc1 (mem_loc (e)));
    }
    if ((Ready = (mem_defined1 (loc = adj_loc1 (mem_loc (e))) &&
                  val_defined (val = get_mem (loc))))) 
       result (val);
}

void reduce_id_ref(e)
NODEPTR e;
{   
    Ready = T;
    if (DebugLevel>2) {
        printf("***reduce_id_ref, NF=%1d, PF=%d, adj_loc=%p\n",NextFlag,PrevFlag,adj_loc1 (mem_loc (e)));
        show_mem_all1(adj_loc1 (mem_loc (e)));
    }
    ref_result (adj_loc1 (mem_loc (e)));
}

void reduce_EMPTY_stmt(e)
NODEPTR e;
{
    put_empty (True);
    Ready = T;
    result (True);
}

void reduce_EMPTY_val(e)
NODEPTR e;
{
    NODEPTR empty_loc = adj_loc1 (EmptyLoc);
    if ((Ready = mem_defined1 (empty_loc)))
        result (get_mem (empty_loc));
}

void reduce_EMPTY_ref(e)
NODEPTR e;
{
    Ready = T;
    ref_result (adj_loc1 (EmptyLoc));
}

void reduce_MORE_stmt(e)
NODEPTR e;
{
    put_empty (False);
    Ready = T;
    result (True);
}

void reduce_MORE_val(e)
NODEPTR e;
{
    NODEPTR e1;

    NODEPTR empty_loc = adj_loc1 (EmptyLoc);
    if ((Ready = mem_defined1 (empty_loc))) {
      e1 = get_mem (empty_loc);
      result (node_bval(e1) ? False : True);
    }
}

/* unit interval */
void reduce_SKIP(e)
NODEPTR e;
{
    put_empty (False);
    Ready = T;
    result (Empty);
}
